#include<reg51.h>
 
#define keyport P1	//keypad connected to P1
#define lcdport P2	//lcd connected to P2
 
sbit C1 = P1^0;     //Column1
sbit C2 = P1^1;     //Column2
sbit C3 = P1^2;     //Column3
sbit C4 = P1^3;     //Column4
 
sbit E    = P3^6;     //E pin for LCD
sbit RS   = P3^7;     //RS pin for LCD
 
//Function declarations
void init_port(void);
void Delay(int);
void LCD_Init(void);
void LCD_Command(int);
void LCD_Data(char);
char Get_key(void);

 
void init_port(void)
{
	P0 = 0x00;   //not used
	P1 = 0x0f;   //used for generating outputs and taking inputs from Keypad
	P2 = 0x00;   //used as data port for LCD
	P3 = 0x00;   //used for RS and E   
}
 
void Delay(int a)
{
   int i;
   for(i=0;i<a;i++);   //Delay
}
 
void LCD_Data(char t)
{
   RS = 1;             // This is data
   P2 = t;             //Data transfer
   E  = 1;             //  E = 1
   Delay(150);
   E  = 0;             // E = 0
   Delay(150);
}
 
 
void LCD_Command(int z)
{
   RS = 0;             // This is command
   P2 = z;             //Data transfer
   E  = 1;             // E = 1
   Delay(150);
   E  = 0;             // E = 0
   Delay(150);
}
 
void LCD_Init(void)
{
 
     Delay(15000);
     LCD_Command(0x30);
     Delay(4500);
     LCD_Command(0x30);
     Delay(300);
     LCD_Command(0x30);
     Delay(650);
 
     LCD_Command(0x38);    //function set
     LCD_Command(0x0c);    //display on,cursor off,blink off
     LCD_Command(0x01);    //clear display
     LCD_Command(0x06);    //entry mode, set increment
}
 
 char get_key()
{
	unsigned char i, k, key = 0;
	k = 1;
	for(i = 0; i < 4; i++){		//loop for 4 rows
		
		keyport &=~ (0x80>>i);	//to make rows low 1 by 1
		 
  		  if(!C1){	            //check if key1 is pressed
			        	Delay(10000);
				        key = k+0;	        //set key number
				        while(!C1);	        //wait for release
				        return key;	        //return key number
			        }
 
			      if(!C2){	             //check if key2 is pressed
				       Delay(10000);
				       key = k+1;	         //set key number
				       while(!C2);	         //wait for release
				        return key;	         //return key number
			         }
 
			         if(!C3){	             //check if key3 is pressed
				        Delay(10000);
				        key = k+2;         	//set key number
				        while(!C3);	        //wait for release
				        return key;	        //return key number
			         }
 
    			      if(!C4){	            //check if key4 is pressed
				        Delay(10000);
				        key = k+3;	        //set key number
				        while(!C4);	        //wait for release
				         return key;	        //return key number
			         }
		k+=4;			              //next row key number
		keyport |= 0x80>>i;	    //make the row high again
	}
	return 0;			            //return 0 if no key pressed
}
 
int main(void)
{
   char key;         // key char for keeping record of pressed key
	
	 init_port();      // Make input and output pins as required
           
        LCD_Init();       // Initilize LCD
 while(1)
{ 
key = Get_key(); // Get pressed key if any
if(key) //only print if a key is pressed
LCD_Data(key+0x60); // Send a printable value to LCD

}
  
}
 